// 12.05.2008
// Jeszcze jedno szukanie grafu 3-e.c. na 21 wierzchokach
// Tym razem C++
// Autor: Przemysaw Gordinowicz
// Pomys: autor+Pawe Praat

#include <stdio.h>
#include <time.h>
#include <string.h>

#define max_order     27
#define max_size      22
#define max_hash_size 799999

typedef unsigned char TEdgeStatus;
typedef signed int TVertex;
typedef unsigned long * TEdgePointer;
typedef signed char * TDegreePointer;
typedef TVertex * TPermPointer;
struct TGraph {
    TEdgePointer e_arr;
    TDegreePointer d1, d2;
};
TGraph CreateGraph(); 
inline void DestroyGraph(TGraph g);
TGraph GraphCopy(const TGraph g);

struct TGraphListEl {
    TGraph graph;
    TGraphListEl * next; 
};

class THashTable {
public:
    unsigned long hash_size;
    unsigned long data_count;
    TGraphListEl ** table;

    THashTable(const int size):
    hash_size(size), data_count(0)
    {
        int i;
        
        table = new TGraphListEl * [hash_size];
        for (i = 0; i < hash_size; i++)
            table[i] = NULL;
    }
    
    ~THashTable() {
        int i;
        TGraphListEl * w;
        for(i = 0; i < hash_size; i++)
            while (table[i]) {
                w = table[i];
                table[i] = table[i]->next;
                DestroyGraph(w->graph);
                delete w;
            }
        delete[] table;
    }

    unsigned long Hashing_F(const TEdgePointer);
    void InsertGraph(const TGraph g);
    void WriteReport(); 
};

//zmienne globalne
TVertex  order;
unsigned short size; // uwaga size <= max_size !
THashTable graphs2ec(17);
int left_min, left_max;
int right_min, right_max;
unsigned long comp_count, insert10_count, insert11_count;

inline TEdgeStatus GetEdgeStatus(const TEdgePointer p, const TVertex u, const TVertex v) {
    int pos = u;
    pos = pos*order - ((1+pos)*pos)/2 + v - 1 - pos;
    return (p[pos / 16] >> (2*(pos % 16))) & 3;
}

inline void SetEdgeStatus(TEdgePointer p, const TVertex u, const TVertex v, const TEdgeStatus s) {
    int pos = u;
    pos = pos*order - ((1+pos)*pos)/2 + v - 1 - pos;
    p[pos/16] = (p[pos/16] & (~(3 << (2*(pos % 16))))) | (s << (2*(pos % 16)));
}

unsigned long THashTable::Hashing_F(const TEdgePointer p) {
    char i;
    unsigned long long r = p[0] % hash_size;
    for(i = 1; i < size; i++)
        r = ((r << 32) + p[i]) % hash_size;
    return (unsigned long) r;
}

char Compare(const TEdgePointer p, const TEdgePointer q) {
    int i;
    for(i = 0; i < size; i++)
        if (p[i] != q[i]) return 0;
    comp_count++;
    return 1;
}

TGraph CreateGraph() {
    TGraph g;

    g.e_arr = new unsigned long[size];
    g.d1 = new signed char[order];
    g.d2 = new signed char[order];
    memset(g.e_arr, 0, size * sizeof(unsigned long));
    memset(g.d1, 0, order);
    memset(g.d2, 0, order);
        
    return g;
}

inline void DestroyGraph(TGraph g) {
    delete[] g.d2;
    delete[] g.d1;
    delete[] g.e_arr;
}

void CopyGraph(const TGraph src, TGraph dest) {
    int i;
    for(i = 0; i < size; i++)
        dest.e_arr[i] = src.e_arr[i];
    for(i = 0; i < order; i++) {
        dest.d1[i] = src.d1[i];
        dest.d2[i] = src.d2[i];
    }
}

TGraph GraphCopy(const TGraph g) {
    TGraph new_g;

    new_g.e_arr = new unsigned long[size];
    new_g.d1 = new signed char[order];
    new_g.d2 = new signed char[order];
    CopyGraph(g, new_g);

    return new_g;    
}

void EmbedGraph(const TGraph src, TGraph dest, const TVertex src_order, const TPermPointer place) {
// osadza graf src w grafie dest na wierzchokach (kolejnych)
// okrelonych przez place[v]
// zakada, e zmieniane krawdzie grafu dest byy na tych
// wierzchokach nieokrelone (0)
    TVertex u, v;
    for(u = 0; u < src_order; u++) {
        for(v = u+1; v < src_order; v++) 
            if (place[u] < place[v])
                SetEdgeStatus(dest.e_arr, place[u], place[v], GetEdgeStatus(src.e_arr, u, v));
            else SetEdgeStatus(dest.e_arr, place[v], place[u], GetEdgeStatus(src.e_arr, u, v));
        dest.d1[place[u]] += src.d1[u];    
        dest.d2[place[u]] += src.d2[u];    
    }
}

void THashTable::InsertGraph(const TGraph g) {
    const unsigned long i = Hashing_F(g.e_arr);
    TGraphListEl * w = table[i];
    
    if (!w) {
        table[i] = new TGraphListEl;
        table[i]->graph = GraphCopy(g);
        table[i]->next = NULL;
        data_count++;
    }
    else {
        while (w->next && !Compare(g.e_arr, w->graph.e_arr))
            w = w->next;
        if (!w->next && !Compare(g.e_arr, w->graph.e_arr)) {
            w->next = new TGraphListEl;
            w->next->graph = GraphCopy(g);
            w->next->next = NULL;
            data_count++;    
        }
    }
}

void PrintGraph(TGraph g) {
    int j;
    for(j = 0; j < size-1; j++)
        printf("%d ", g.e_arr[j]);
    printf("%d\n", g.e_arr[size-1]);
    for(j = 0; j < order-1; j++)
        printf("%d ", g.d1[j]);
    printf("%d\n", g.d1[order-1]);
    for(j = 0; j < order-1; j++)
        printf("%d ", g.d2[j]);
    printf("%d\n", g.d2[order-1]);    
}

void THashTable::WriteReport() {  
    unsigned long i, j;
    TGraphListEl * w;
    FILE * out_file = fopen("report.txt", "at");

    for(i = 0, w = table[i]; i < hash_size; w = table[++i])
        while (w) {
            for(j = 0; j < size-1; j++)
                fprintf(out_file, "%d ", w->graph.e_arr[j]);
            fprintf(out_file, "%d\n", w->graph.e_arr[size-1]);
            for(j = 0; j < order-1; j++)
                fprintf(out_file, "%d ", w->graph.d1[j]);
            fprintf(out_file, "%d\n", w->graph.d1[order-1]);
            for(j = 0; j < order-1; j++)
                fprintf(out_file, "%d ", w->graph.d2[j]);
            fprintf(out_file, "%d\n", w->graph.d2[order-1]);
            w = w->next;
        }
    fclose(out_file);
}

void EmbedIsoGraph9(TGraph dest, const TVertex vert, const TEdgeStatus st, const TVertex z0, const TVertex z1,
                    const TPermPointer p, THashTable * table) {
// osadza (po kolei) wszystkie grafy izomorficzne z G_9
// w grafie dest na wierzchokach okrelonych przez (p1[v] v=0..z1-1)
// i dobraniu pozostaych wierzchokw z p1[v] v=z1..z1+z0-1
// o ile to jest moliwe (zmieniane s tylko krawdzie nieokrelone (0)
// wierzchoki z p0 ktre wchodz do grafu na poczeniu z wierzchokiem
// vert dostaja status st a te ktre nie wchodz status (st mod 2) + 1
// wane z1 >= 1
    if (z1 + z0 >= 9) {
        TGraph g = CreateGraph();
        TVertex a, u, v, x, y;
        TVertex spec;
        char q;
        TEdgeStatus s1, s2;

        TPermPointer place = new TVertex[order];
        TPermPointer rev = new TVertex[order];
		for(u = 0; u < order; u++) 
			place[u] = rev[u] = -1;
		
        TGraph src = graphs2ec.table[0]->graph;

        // specjalne traktowanie - jeli mam par poczonych wierzchokw to
        // wstawiam na je na sztywno na 0,1. Jeli dodatkowo mam wierzchoek nie ssiadujcy
        // z 1 ani 2 to wstawiam go na 5
        q = 0; x = y = 0;
        for(u = 0; u < z1 - 1; u++) {
            for(v = u+1; v < z1; v++) {
                if ((p[u] < p[v] ? GetEdgeStatus(dest.e_arr, p[u], p[v]) : GetEdgeStatus(dest.e_arr, p[v], p[u])) == 1) {
                    x = u;   // zapamitujemy par wierzchokw poczonych
                    y = v;
                    // i szukamy trzeciego niepoczonego
                    for(a = 0; a < z1; a++)
                        if ((a != u) && (a != v) &&
                                ((p[a] < p[u] ? GetEdgeStatus(dest.e_arr, p[a], p[u]) : GetEdgeStatus(dest.e_arr, p[u], p[a])) == 2) &&
                                ((p[a] < p[v] ? GetEdgeStatus(dest.e_arr, p[a], p[v]) : GetEdgeStatus(dest.e_arr, p[v], p[a])) == 2)) {
                            q = 1;
                            break;
                        }
                }
                if (q) break;
            }
            if (q) break;
        }
        if (q) { // mamy trjk (u, v, a)
            // zamiany
            x = p[0]; p[0] = p[u]; p[u] = x;
            x = p[1]; p[1] = p[v]; p[v] = x;
            if (a == 0) a = u;  // a mogo zmieni miejsce
            if (a == 1) a = v;
            x = p[2]; p[2] = p[a]; p[a] = x;
            place[0] = p[0]; rev[0] = 0;
            place[1] = p[1]; rev[1] = 1;
            place[5] = p[2]; rev[2] = 5;
            spec = 2;
        }
        else if (y) {       // mam par (x, y)
            // zamiany
            u = p[0]; p[0] = p[x]; p[x] = u;
            u = p[1]; p[1] = p[y]; p[y] = u;
            place[0] = p[0]; rev[0] = 0;
            place[1] = p[1]; rev[1] = 1;
            spec = 1;
        }
        else {              // nie mam nic (czyli mam jeden wierzcholek)
            place[0] = p[0]; rev[0] = 0;
            spec = 0;            
        }

        a = spec+1;
        do {        // rozoy wierzchoki z p1 i p0 w place tak eby byo ok
            while ((a < z1) && (rev[spec] != -1)) {
                if (rev[a] != -1) place[rev[a]] = -1;
                u = rev[a] + 1;
                do {
                    while (place[u] != -1) u++;
                    // czy p[a] moe by na pozycji u?
                    q = 0;
                    if (u < 9)
                        for (v = 0; v < 9; v++)
                            if (place[v] != -1) {
                                s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : GetEdgeStatus(src.e_arr, v, u);
                                s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : GetEdgeStatus(dest.e_arr, place[v], p[a]);
                                q = (s2 > 0) && (s1 != s2);
                                if (q) break;
                            }
                    if (q) u++;    
                } while (q && (u < 9));
                if (u >= 9) {
                    rev[a] = -1;
                    a--;
                    if (a == spec) rev[spec] = -1; // koniec - trzeba by zmnieni to co ustalone
                }
                else {
                    place[u] = p[a];
                    rev[a] = u;
                    a++;
                }
            }   

            if (rev[spec] != -1) {      // dobrano wierzchoki "obowizkowe" - czas na potencjalne
                if (z0 > 0) do {
                    if (rev[a] != -1) place[rev[a]] = -1;
                    u = rev[a] + 1;
                    do {
                        while (place[u] != -1) u++;
                        // czy p[a] moe by na pozycji u ?
                        q = 0;
                        if (u < 9)
                            for (v = 0; v < 9; v++)
                                if (place[v] != -1) {
                                    s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : GetEdgeStatus(src.e_arr, v, u);
                                    s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : GetEdgeStatus(dest.e_arr, place[v], p[a]);
                                    q = (s2 > 0) && (s1 != s2);
                                    if (q) break;
                                }
                        if (q) u++;    
                    } while (q && (u < 10) && ((u != 9) || (place[9] != -1)));
                    if (u >= z0 + z1) {
                        rev[a] = -1;
                        a--;
                        while (rev[a] >= 9) {
                            place[rev[a]] = -1;
                            rev[a] = -1;
                            a--;
                        }
                    }
                    else {
                       place[u] = p[a];
                       rev[a] = u;
                       a++; 
                    }
                } while ((a < z0 + z1) && (rev[z1] != -1));
                if (a >= z0 + z1) a = z0 + z1 -1;
                if (z0 == 0 || rev[z1] != -1) {
                    // Wstawi graf
                    CopyGraph(dest, g);
                    q = 0;
                    for(u = 0; (u < 8) && !q; u++) {
                        for(v = u+1; (v < 9) && !q; v++) {
                            if (place[u] < place[v]) {
                                x = place[u];
                                y = place[v];
                            }
                            else {
                                x = place[v];
                                y = place[u];                            
                            }
							if (!GetEdgeStatus(g.e_arr, x, y)) {
							    s1 = GetEdgeStatus(src.e_arr, u, v);
                                SetEdgeStatus(g.e_arr, x, y, s1);
                                q = (s1 == 1) ? ((++(g.d1[x]) > 11) || (++(g.d1[y]) > 11)) : ((++(g.d2[x]) > 11) || (++(g.d2[y]) > 11));
                            }
                        }
                    }
                    if (!q) {
                        for(v = 0; (v < z0) && !q; v++) {
                            s1 = rev[z1+v] < 9 ? st : (st % 2) + 1;
                            if (vert < p[z1+v])
                                SetEdgeStatus(g.e_arr, vert, p[z1+v], s1);
                            else SetEdgeStatus(g.e_arr, p[z1+v], vert, s1);
                            q = (s1 == 1) ? (++(g.d1[vert]),++(g.d1[p[z1+v]]) > 11) : (++(g.d2[vert]),++(g.d2[p[z1+v]]) > 11);
                        }
                        if (!q) {
                            table->InsertGraph(g);
                            if (st == 2) insert11_count++;
                        }
                    }
                }
            }
        } while (rev[spec] != -1);
        
        delete[] rev;
        delete[] place;
        DestroyGraph(g);
    }
}

void EmbedIsoGraph10(TGraph dest, const TVertex vert, const TEdgeStatus st, const TVertex z0, const TVertex z1,
                     const TPermPointer p, THashTable * table) {
// osadza (po kolei) wszystkie grafy izomorficzne z kolejnymi G_10
// w grafie dest na wierzchokach okrelonych przez (p1[v] v=0..z1-1)
// i dobraniu pozostaych wierzchokw z p1[v] v=z1..z1+z0-1
// o ile to jest moliwe (zmieniane s tylko krawdzie nieokrelone (0)
// wierzchoki z p0 ktre wchodz do grafu na poczeniu z wierzchokiem
// vert dostaja status st a te ktre nie wchodz status (st mod 2) + 1
// wane z1 >= 1
    if (z1 + z0 >= 10) {
        TGraph g = CreateGraph();
        TVertex a, u, v, x, y;
        char q;
        TEdgeStatus s1, s2;
        int i;
        
        TPermPointer place = new TVertex[order];
        TPermPointer rev = new TVertex[order];
		for(u = 0; u < order; u++) 
			place[u] = rev[u] = -1;

        for (i = 1; i < 17; i++) {
            TGraph src = graphs2ec.table[i]->graph;

            place[0] = p[0];
            rev[0] = 0;
            a = 1;
            do {        // rozoy wierzchoki z p1 i p0 w place tak eby byo ok
                while ((a < z1) && (rev[0] != -1)) {
                    if (rev[a] != -1) place[rev[a]] = -1;
                    u = rev[a] + 1;
                    do {
                        while (place[u] != -1) u++;
                        // czy p[a] moe by na pozycji u?
                        q = 0;
                        if (u < 10)
                            for (v = 0; v < 10; v++)
                                if (place[v] != -1) {
                                    s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : GetEdgeStatus(src.e_arr, v, u);
                                    s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : GetEdgeStatus(dest.e_arr, place[v], p[a]);
                                    q = (s2 > 0) && (s1 != s2);
                                    if (q) break;
                                }
                        if (q) u++;    
                    } while (q && (u < 10));
                    if (u >= 10) {
                        rev[a] = -1;
                        a--;
                    }
                    else {
                        place[u] = p[a];
                        rev[a] = u;
                        a++;
                    }
                }   

                if (rev[0] != -1) {      // dobrano wierzchoki "obowizkowe" - czas na potencjalne
                    if (z0 > 0) do {
                        if (rev[a] != -1) place[rev[a]] = -1;
                        u = rev[a] + 1;
                        do {
                            while (place[u] != -1) u++;
                            // czy p[a] moe by na pozycji u ?
                            q = 0;
                            if (u < 10)
                                for (v = 0; v < 10; v++)
                                    if (place[v] != -1) {
                                        s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : GetEdgeStatus(src.e_arr, v, u);
                                        s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : GetEdgeStatus(dest.e_arr, place[v], p[a]);
                                        q = (s2 > 0) && (s1 != s2);
                                        if (q) break;
                                    }
                            if (q) u++;    
                        } while (q && (u < 11) && ((u != 10) || (place[10] != -1)));
                        if (u >= z0 + z1) {
                            rev[a] = -1;
                            a--;
                            while (rev[a] >= 10) {
                                place[rev[a]] = -1;
                                rev[a] = -1;
                                a--;
                            }
                        }
                        else {
                            place[u] = p[a];
                            rev[a] = u;
                            a++; 
                        }
                    } while ((a < z0 + z1) && (rev[z1] != -1));
                    if (a >= z0 + z1) a = z0 + z1 -1;
                    if (z0 == 0 || rev[z1] != -1) {
                        // Wstawi graf
                        CopyGraph(dest, g);
                        q = 0;
                        for(u = 0; (u < 9) && !q; u++) {
                            for(v = u+1; (v < 10) && !q; v++) {
                                if (place[u] < place[v]) {
                                    x = place[u];
                                    y = place[v];
                                }
                                else {
                                    x = place[v];
                                    y = place[u];                            
                                }
								if (!GetEdgeStatus(g.e_arr, x, y)) {
									s1 = GetEdgeStatus(src.e_arr, u, v);
									SetEdgeStatus(g.e_arr, x, y, s1);
									q = (s1 == 1) ? ((++(g.d1[x]) > 11) || (++(g.d1[y]) > 11)) : ((++(g.d2[x]) > 11) || (++(g.d2[y]) > 11));
								}
							}
						}
						if (!q) {
							for(v = 0; (v < z0) && !q; v++) {
								s1 = rev[z1+v] < 10 ? st : (st % 2) + 1;
								if (vert < p[z1+v])
									SetEdgeStatus(g.e_arr, vert, p[z1+v], s1);
								else SetEdgeStatus(g.e_arr, p[z1+v], vert, s1);
								q = (s1 == 1) ? (++(g.d1[vert]),++(g.d1[p[z1+v]]) > 11) : (++(g.d2[vert]),++(g.d2[p[z1+v]]) > 11);
							}
                            if (!q) {
                                table->InsertGraph(g);
                                if (st == 2) insert10_count++;
                            }
                        }
                    }
                }
            } while (rev[0] != -1);
        }
        DestroyGraph(g);
        delete[] rev;
        delete[] place;
    }
}

void ReadGraphs2ec() {
    FILE * in_file;
    char graph_text[100];
    char file_name[20];
    char * c;
    int i, order;
    TVertex u, v, bit, number;

    for (i = 0, order = 9; order <= 10; order++) {
        sprintf(file_name, "%d_2ec.txt", order);
        in_file = fopen(file_name, "rt");
        for ( ; !feof(in_file); i++)
        {
            // czytamy graf - format Brandana McKay'a; ilo wierzchokw to order
            fscanf(in_file, "%s\n", graph_text);
            c = graph_text+1;

            graphs2ec.table[i] = new TGraphListEl;
            graphs2ec.table[i]->next = NULL;
            graphs2ec.table[i]->graph = CreateGraph();
            for (u = 0; u < order; u++) {
                graphs2ec.table[i]->graph.d1[u] = 0;
                graphs2ec.table[i]->graph.d2[u] = order-1;
                for (v = u+1; v < order; v++)
                    SetEdgeStatus(graphs2ec.table[i]->graph.e_arr, u, v, 2);
            }
            bit = 5;
            number = *c-63;
            c++;

            for (u = 1 ; u < order ; u++)
            {
                for (v = 0 ; v < u ; v++)
                {
                    if (bit == -1)
                    {
                        bit = 5;
                        number = *c-63;
                        c++;
                    }

                    if ((number >> bit) & 1) {
                        graphs2ec.table[i]->graph.d1[u]++;
                        graphs2ec.table[i]->graph.d1[v]++;
                        graphs2ec.table[i]->graph.d2[u]--;
                        graphs2ec.table[i]->graph.d2[v]--;
                        SetEdgeStatus(graphs2ec.table[i]->graph.e_arr, v, u, 1);
                    }
                    bit--;
                }
            }
        }
        fclose(in_file);
    }
}

THashTable * EmbedBFS(const TGraph g, const TVertex cr) {
// Obsadza najpierw cr "9" na {12..21}
// wiadomo, e 2 <= cr <= 8
// poprzez przeszukiwanie wszerz dobiera brakujce krawdzie
// na wejciu graf z dobranymi wszyskimi ssiadami 1
// do dobrania {2..11}x{12..21}
    int i, j;
    TVertex u, v, x, z0, z1, z2;
    THashTable * (graphs[11]);
    THashTable * temp;
    TGraphListEl * w;
    TGraph g0;
    TPermPointer p0, p1, p2, p3;
    unsigned long int hash_size[5] = {1, 167, 3299, 5999, 5999};

    p0 = new TVertex[order];
    p1 = new TVertex[order];
    p2 = new TVertex[order];
    p3 = new TVertex[order];

    for (u = 0; u < 11; u++)
        graphs[u] = NULL;
    graphs[0] = new THashTable(1);
    graphs[0]->InsertGraph(g);
    graphs[cr] = new THashTable(max_hash_size);

    // najpierw cr '9'
    // do ustalenia mamy cr wierzchokw z {12..21} ktre bd
    // stopnia 9-tego (a wszystko zbiera graphs[cr+1])
    for (v = 0; v < cr; v++)
         p3[v] = v;
    do {
        if (p3[cr-1] > 9) {
            p3[cr-1] = 9;
            for (v = cr - 2; p3[v] == p3[v+1] -1; v--) 
                if (graphs[v+1]) {
                    delete graphs[v+1];
                    graphs[v+1] = NULL;
                }
            if (graphs[v+1]) {
                delete graphs[v+1];
                graphs[v+1] = NULL;
            }
            
            p3[v]++;
            for (; v < cr - 1; v++)
                p3[v+1] = p3[v] + 1;
        }
        while (p3[cr-1] < 10) { //p3 ustalone
            for(u = 0; u < cr; u++) {
                if (graphs[u] && graphs[u]->data_count && ((u == cr-1) || (!graphs[u+1]))) {
                    if (u < cr-1) 
                        graphs[u+1] = new THashTable(u < 4 ? hash_size[u+1] : max_hash_size);
                    x = 11 + p3[u]; // rozwaamy wierzchoek x
                    for (i = 0; i < graphs[u]->hash_size; i++) {
                        for(w = graphs[u]->table[i]; w; w = w->next) {
                            g0 = w->graph;
                            z0 = z1 = 0;
                            for (v = 0; v < x; v++) 
                                switch (GetEdgeStatus(g0.e_arr, v, x)) {
                                    case 0: p0[z0++] = v; break;
                                    case 1: p1[z1++] = v;
                                }
                            for (v = x+1; v < 21; v++)
                                switch (GetEdgeStatus(g0.e_arr, x, v)) {
                                    case 0: p0[z0++] = v; break;
                                    case 1: p1[z1++] = v;
                                }
                            for (v = 0; v < z0; v++)
                                p1[z1+v] = p0[v];
                            EmbedIsoGraph9(g0, x, 1, z0, z1, p1, graphs[u+1]); // otoczenie gdy x ma mie stopie 9
                        }
                   }
                }
                if (graphs[u+1] && graphs[u+1]->data_count > graphs[u+1]->hash_size)
                {
                    time_t t = time(NULL);
                    printf("%d: %d \t\t %.8s\n", u+1, graphs[u+1]->data_count, ctime(&t)+11);
                }
            }
            p3[cr-1]++;
        }
    } while (p3[0] < 10 - cr);

    for (u = 0; u < cr; u++)
        if (graphs[u]) {
            delete graphs[u];
            graphs[u] = NULL;
        }

    graphs[0] = graphs[cr];
    graphs[cr] = NULL;

    if (graphs[0]->data_count) {
        time_t t = time(NULL);
        printf("0(%d): %d \t\t %.8s\n", cr, graphs[0]->data_count, ctime(&t)+11);
    }
    // teraz dowolne stopnie wierzchokw 1..10
    for(u = 1; u <= 10; u++) {
        if (graphs[u-1] && graphs[u-1]->data_count) {
            graphs[u] = new THashTable(max_hash_size);
            for (i = 0; i < graphs[u-1]->hash_size; i++) {
                while (graphs[u-1]->table[i]) {
                    w = graphs[u-1]->table[i];
                    graphs[u-1]->table[i] = graphs[u-1]->table[i]->next;
                    g0 = w->graph;
                    delete w;
                    z0 = z1 = z2 = 0;
                    for (v = 0; v < u; v++)
                        if (GetEdgeStatus(g0.e_arr, v, u) == 1)
                            p1[z1++] = v;
                        else p2[z2++] = v;
                    for (v = u+1; v < 21; v++)
                        switch (GetEdgeStatus(g0.e_arr, u, v)) {
                            case 0: p0[z0++] = v; break;
                            case 1: p1[z1++] = v; break;
                            default: p2[z2++] = v;       // case 2
                        }
                    for (v = 0; v < z0; v++)
                        p1[z1+v] = p2[z2+v] = p0[v];

                    EmbedIsoGraph9(g0, u, 1, z0, z1, p1, graphs[u]); // otoczenie, gdy u ma mie stopie 9
                    EmbedIsoGraph9(g0, u, 2, z0, z2, p2, graphs[u]); // przeciwotoczenie, gdy u ma mie stopie 11
                    // otoczenie i przeciwotoczenie, gdy u ma mie stopie 10
                    temp = new THashTable(19999);
                    EmbedIsoGraph10(g0, u, 1, z0, z1, p1, temp);
                    if (temp->data_count) {
                        for (j = 0; j < temp->hash_size; j++)
                            while (temp->table[j]) {
                                w = temp->table[j];
                                temp->table[j] = temp->table[j]->next;
                                z2 = 0;
                                for (v = 0; v < u; v++)
                                    if (GetEdgeStatus(w->graph.e_arr, v, u) == 2)
                                        p2[z2++] = v;
                                for (v = u+1; v < 21; v++)
                                    if (GetEdgeStatus(w->graph.e_arr, u, v) == 2)
                                        p2[z2++] = v;
                                EmbedIsoGraph10(w->graph, u, 2, 0, z2, p2, graphs[u]);
                                DestroyGraph(w->graph);
                                delete w;        
                            }
                    }
                    delete temp;
					temp = NULL;
                    DestroyGraph(g0);
                }
            }
        }
        if (graphs[u]) {
            time_t t = time(NULL);
            printf("%d: %d \t\t %.8s\n", u, graphs[u]->data_count, ctime(&t)+11);
        }
		if (graphs[u-1]) {
			delete graphs[u-1];
			graphs[u-1] = NULL;
		}
    }

    delete [] p3;
    delete [] p2;
    delete [] p1;
    delete [] p0;
    
    return graphs[10];
}

void Find3ecCandidate21() {
    int i, j, cl, cr;
    TVertex u;
    TPermPointer p1, p2;
    TGraph g0, g1, g2;
    THashTable * table;

    p1 = new TVertex[order];
    p2 = new TVertex[order];

    // graf wzorcowy
    g0 = CreateGraph();
    g1 = CreateGraph();
    g2 = CreateGraph();
    // wierzchoek "0" poczony z 10-ma wierzchokami
    for (u = 1; u < 11; u++) {
        p1[u-1] = u;
        SetEdgeStatus(g0.e_arr, 0, u, 1);
        g0.d1[u]++;
    }
    g0.d1[0] = 10;
    // i nie poczony z 10-ma
    for (u = 11; u < 21; u++) {
        p2[u-11] = u;
        SetEdgeStatus(g0.e_arr, 0, u, 2);
        g0.d2[u]++;
    }
    g0.d2[0] = 10;

    for (i = left_min; i <= left_max; i++) {
        CopyGraph(g0, g1);
        EmbedGraph(graphs2ec.table[i]->graph, g1, 10, p1);
        cl = 0;
        for (u = 0; u < 10; u++)
            cl += graphs2ec.table[i]->graph.d1[u];
        cl = cl / 2;
        for (j = right_min; j <= right_max; j++) {
			CopyGraph(g1, g2);
            printf("%d %d\n", i, j); //comp_count

            EmbedGraph(graphs2ec.table[j]->graph, g2, 10, p2);
            cr = 0;
            for (u = 0; u < 10; u++)
                cr += graphs2ec.table[j]->graph.d1[u];
            cr = cr / 2;
            // 5 + cl - cr '9' w R
            table = EmbedBFS(g2, 5 + cl - cr);
            if (table) {
                if (table->data_count) table->WriteReport();
                delete table;
            }
        }
    }
    DestroyGraph(g2);
    DestroyGraph(g1);
    DestroyGraph(g0);

    delete [] p1;
    delete [] p2;    
}

int main(int argc, char * argv[])
{
    time_t t = time(NULL);
    printf("%.8s\n", ctime(&t)+11);

    order = 21;
    size = order; 
    size = (((size * (size-1)) / 2) + 15) / 16;

    ReadGraphs2ec();

    if (argc > 1) sscanf(argv[1], "%d", &left_min);
    else left_min = 1;
    if (argc > 2) sscanf(argv[2], "%d", &left_max);
    else left_max = 16;
    if (argc > 3) sscanf(argv[3], "%d", &right_min);
    else right_min = 1;
    if (argc > 4) sscanf(argv[4], "%d", &right_max);
    else right_max = 16;

    comp_count = 0;
    Find3ecCandidate21();

    t = time(NULL);
    printf("%d\t%d\t%d\t%.8s\n", comp_count, insert10_count, insert11_count, ctime(&t)+11);
   
    return 0;
}
